local ini = require "formats.ini";
local xml = require "formats.xml";

local flatpak = {};

--- @param repo ostree
local function get_metainfo(repo, commit, metadata)
	local root_dir = repo:read_dir(commit.root);

	local export_dir = root_dir:dir(repo, "export");
	if export_dir == nil then return nil end

	local share_dir = export_dir:dir(repo, "share");
	if share_dir == nil then return nil end

	local metainfo_dir = share_dir:dir(repo, "metainfo");
	if metainfo_dir == nil then return nil end

	local metainfo_ref =
		metainfo_dir.files[metadata.Application.name .. ".metainfo.xml"] or
		metainfo_dir.files[metadata.Application.name .. ".appdata.xml"];
	if metainfo_ref == nil then return nil end

	local metainfo_f = repo:read_file(metainfo_ref);

	local parts = {};
	for el in metainfo_f:get_content() do parts[#parts + 1] = el end
	print(table.concat(parts, ""));
	return xml.parse(table.concat(parts, ""));
end

--- @param repo ostree
function flatpak.parse_package(repo, full_name)
	local ref = repo:read_ref(full_name);
	local commit = repo:read_commit(ref);
	local name, arch, branch = full_name:match "^.-/(.-)/(.-)/(.-)$";
	print(arch, branch);

	local metadata = ini.parse(commit.metadata["xa.metadata"]);
	local metainfo = get_metainfo(repo, commit, metadata);

	local res = {};

	pprint(metadata);

	res.ref = full_name;
	res.name = name;
	res.arch = arch;
	res.branch = branch;

	res.size = commit.metadata["xa.download-size"];
	res.command = metadata.Application.command;
	res.runtime = metadata.Application.runtime;
	res.deps = {};

	for key, map in pairs(metadata) do
		local dep = key:match "^Extension%s+(.-)%s*$";
		if dep then
			dep = (dep .. "@"):match "^(.-)@";

			local dep_ref = "runtime/" .. dep .. "/" .. arch .. "/" .. (map.version or branch);
			local required = map["no-autodownload"] ~= "true";
			local mounts = {};

			if map.directory then
				mounts[#mounts + 1] = map.directory;
			elseif map.directories then
				mounts = ini.parse_list(map.directories);
			end

			if dep:find "%.Debug$" then
				res.deps.debug = {
					name = dep_ref,
					required = required,
					mounts = mounts,
				};
			elseif dep:find "%.Locale$" then
				res.deps.locale = {
					name = dep_ref,
					required = required,
					mounts = mounts,
				};
			else
				res.deps[#res.deps + 1] = {
					name = dep_ref,
					required = required,
					mounts = mounts,
				};
			end

			pprint(dep, map);
		end
	end

	if metainfo then
		local app_meta = metainfo:get "component";
		res.version = app_meta:get "releases":get_all "release"[1].attribs.version;
		res.license = app_meta:get "project_license":text();
		res.summary = app_meta:get_all "summary"[1]:text();
	end

	return res, commit;
end

return flatpak;
