--- @param consumers table<string, (fun(arg: string): boolean?, boolean?) | string>
--- @param ... string
--- @returns nil
return function (consumers, ...)
	local consumer_stack = {};
	local pass_args = false;

	local function digest_arg(v)
		while true do
			--- @type (fun(arg: string): boolean?, boolean?)?
			local el = table.remove(consumer_stack);
			if el == nil then break end

			local res, fin = el(v);
			if fin then
				pass_args = true;
			end

			if fin or res then
				return;
			end
		end

		while true do
			local res, fin = consumers[1](v);
			if fin then
				pass_args = true;
			end

			if res or fin then
				return;
			end
		end
	end

	local function get_consumer(name)
		local consumer = name;
		local path = {};
		local n = 0;

		while true do
			local curr = consumer;
			if path[curr] then
				local path_arr = {};

				for k, v in next, path do
					path_arr[v] = k;
				end

				error("Alias to '" .. curr .. "' is recursive: " .. table.concat(path_arr, " -> "));
			end

			consumer = consumers[curr];
			if consumer == nil then
				local prefix;
				if n == 0 then
					prefix = "Unknown flag";
				else
					prefix = "Unknown alias";
				end

				if #curr == 1 then
					error(prefix .. " '-" .. curr .. "'");
				else
					error(prefix .. " '--" .. curr .. "'");
				end
			elseif type(consumer) == "function" then
				return consumer;
			end

			path[curr] = n;
			n = n + 1;
		end
	end

	local function next(...)
		if ... == nil then
			while #consumer_stack > 0 do
				local el = table.remove(consumer_stack);

				if el(nil) then
					error("Unexpected end of arguments", 0);
					break;
				end
			end
		else
			if pass_args then
				consumers[1]((...));
			elseif ... == "--" then
				pass_args = true;
			elseif (...):match "^%-%-" then
				consumer_stack[#consumer_stack + 1] = get_consumer((...):sub(3));
			elseif (...):match "^%-" then
				for c in (...):sub(2):gmatch "." do
					table.insert(consumer_stack, 1, get_consumer(c));
				end
			else
				digest_arg(...);
			end

			return next(select(2, ...));
		end
	end

	return next(...);
end
